
#include "usart_lib.h"

// Initialize USART Module
void USART_Init(void)
{

	//Enable Transmitter and receiver
	UCSR0B |= (1<<RXCIE0)|(1 << TXEN0)|(1 << RXEN0);
	
	// Load lower 8-bits of the baud rate value into the low byte of the UBRR register 
	UBRR0L = BAUD_PRESCALE;
	// Load upper 8-bits of the baud rate value into the high byte of the UBRR register 
	UBRR0H = (BAUD_PRESCALE >> 8);
	
	//Set data frame format: asynchronous mode,no parity, 1 stop bit, 8 bit size
	// This is set by default in ATtiny2313, so not really necessary
	UCSR0C = (0<<UMSEL00)|(0<<UPM01)|(0<<UPM00)|(0<<USBS0)|(1<<UCSZ01)|(1<<UCSZ00);
	
	// Initialize the buffer
	writeIndex = 0;
	currentIndex = 0;
}

// Send a single byte over USART
void USART_SendByte(uint8_t data)
{
	// Wait if a byte is being transmitted
	while((UCSR0A & (1<<UDRE0)) == 0);

	// Transmit LSBs of data
	UDR0 = data;
}

uint8_t popFromBuffer(void)
{
	uint8_t temp;
	
	// Pop a byte from the stack, in an atomic action
	cli();
	temp = usartBuffer[currentIndex];
	currentIndex++;
	if (currentIndex >= BUFFERSIZE)
	{
		currentIndex = 0;
	}
	sei();
	
	return temp;
}

uint8_t peekFromBuffer(void)
{
	uint8_t temp;
	
	// Read a byte from the stack, but don't change the read indicators, in an atomic action
	cli();
	temp = usartBuffer[currentIndex];
	sei();
	
	return temp;
}

void bufferInsert(uint8_t data)
{
	// This must be done atomically
	cli();
	
	// Store new data in the buffer
	usartBuffer[writeIndex] = data;
	writeIndex++;
	if (writeIndex >= BUFFERSIZE)
	{
		writeIndex = 0;
	}
	
	sei();
}

// Send a character string over serial. Automatically appends a line feed '\n' once done.
void sendSerialString(char stringData[])
{
	uint8_t i = 0;
	char currentChar;
	
	// Make string writes atomic
	cli();
	
	// Send each char until reaching a null termination
	currentChar = stringData[i];
	while(currentChar != '\0')
	{
		USART_SendByte(currentChar);
		i++;
		currentChar = stringData[i];
	}
	
	// Send the '\n' line feed character
	USART_SendByte('\n');
	
	sei();
}
